// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.io.*;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.awt.Component;

import javax.swing.JOptionPane;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.VelocityException;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.fileEditor.OpenFileDescriptor;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.openapi.module.Module;
import com.intellij.psi.*;

import com.kiwisoft.generate.model.Template;
import com.kiwisoft.generate.model.TemplateFile;
import com.kiwisoft.generate.model.TemplateManager;
import com.kiwisoft.utils.idea.PluginUtils;
import com.kiwisoft.utils.velocity.VelocityUtils;
import com.kiwisoft.utils.gui.Icons;
import com.kiwisoft.utils.StringUtils;

@SuppressWarnings({"ComponentNotRegistered"})
public class GenerateAction extends AnAction
{
	private Template template;

	public GenerateAction(Template template)
	{
		super(template.getName(), template.getDescription(), Icons.getIcon("generate"));
		this.template=template;
	}

	public void update(AnActionEvent e)
	{
		super.update(e);
		DataContext dataContext=e.getDataContext();
		VirtualFile virtualDirectory=(VirtualFile)dataContext.getData(DataConstants.VIRTUAL_FILE);
		Project project=(Project)dataContext.getData(DataConstants.PROJECT);
		boolean enabled=virtualDirectory!=null && project!=null;
		if (enabled && template.isRequiresSourceFolder())
		{
			PsiElement psiElement=(PsiElement)dataContext.getData(DataConstants.PSI_ELEMENT);
			enabled=getPsiPackage(psiElement)!=null;
		}
		e.getPresentation().setEnabled(enabled);
	}

	private PsiPackage getPsiPackage(PsiElement psiElement)
	{
		if (psiElement instanceof PsiDirectory)
		{
			return ((PsiDirectory)psiElement).getPackage();
		}
		else if (psiElement instanceof PsiFile)
		{
			return getPsiPackage(((PsiFile)psiElement).getContainingDirectory());
		}
		else if (psiElement!=null)
		{
			return getPsiPackage(psiElement.getContainingFile());
		}
		return null;
	}

	private PsiDirectory getPsiDirectory(PsiElement psiElement)
	{
		if (psiElement instanceof PsiDirectory)
		{
			return (PsiDirectory)psiElement;
		}
		else if (psiElement instanceof PsiFile)
		{
			return ((PsiFile)psiElement).getContainingDirectory();
		}
		else if (psiElement!=null)
		{
			return getPsiDirectory(psiElement.getContainingFile());
		}
		return null;
	}

	private PsiDirectory getPsiRootDirectory(PsiElement element)
	{
		PsiDirectory psiDirectory=getPsiDirectory(element);
		if (psiDirectory==null || psiDirectory.getPackage()==null) return null;
		while (true)
		{
			PsiDirectory parentDirectory=psiDirectory.getParentDirectory();
			if (parentDirectory==null || parentDirectory.getPackage()==null) return psiDirectory;
			psiDirectory=parentDirectory;
		}
	}

	public void actionPerformed(AnActionEvent event)
	{
		DataContext dataContext=event.getDataContext();
		VirtualFile directory=(VirtualFile)dataContext.getData(DataConstants.VIRTUAL_FILE);
		final Project project=(Project)dataContext.getData(DataConstants.PROJECT);
		if (directory!=null && project!=null)
		{
			Object parameterBean=null;
			if (!template.getParameters().isEmpty())
			{
				ParameterDialog dialog=new ParameterDialog(project, template);
				PluginUtils.showDialog(dialog, true, true);
				if (dialog.isOK()) parameterBean=dialog.getBean();
			}
			else
			{
				try
				{
					parameterBean=template.createDynaClass().newInstance();
				}
				catch (Exception e)
				{
					handleThrowable(e, dataContext);
				}
			}
			if (parameterBean!=null)
			{
				if (!directory.isDirectory()) directory=directory.getParent();
				final VelocityEngine velocity=new VelocityEngine();
				try
				{
					velocity.init();
					final VelocityContext context=VelocityUtils.createContext();
					initializeStaticParameters(context);
					context.put("data", parameterBean);
					context.put("Selection", getSelectionData(dataContext, directory));
					List<String> resultMessages=new ArrayList<String>();
					if (template.getFiles().isEmpty()) resultMessages.add("No template files defined.");
					for (final TemplateFile templateFile : template.getFiles())
					{
						String fileNamePattern=templateFile.getFileName();
						String fileName=evaluate(velocity, context, fileNamePattern);
						File file=new File(fileName);
						if (!file.isAbsolute()) file=new File(VfsUtil.virtualToIoFile(directory), fileName);
						if (file.exists() && !templateFile.isOverwrite())
						{
							resultMessages.add("Skipped creation of file '"+file.getAbsolutePath()+"' because it already exists.");
							continue;
						}
						else
						{
							try
							{
								if (!file.getParentFile().exists())
								{
									file.getParentFile().mkdirs();
								}
								FileWriter fileWriter=new FileWriter(file);
								velocity.evaluate(context, fileWriter, GenerateAction.class.getName(), templateFile.getContent());
								fileWriter.close();
								resultMessages.add("Successfully created file '"+file.getAbsolutePath()+"'.");
								refreshFile(project, templateFile, file);
							}
							catch (Exception e)
							{
								resultMessages.add("Error while creating file '"+file.getAbsolutePath()+"'.");
								handleThrowable(e, dataContext);
							}
						}
					}
					JOptionPane.showMessageDialog(WindowManager.getInstance().suggestParentWindow(project),
												  StringUtils.enumerate(resultMessages, "\n"),
												  "Result", JOptionPane.INFORMATION_MESSAGE);
				}
				catch (Exception e)
				{
					handleThrowable(e, dataContext);
				}
			}
		}
	}

	private void initializeStaticParameters(VelocityContext context)
	{
		Map<String, String> parameters=TemplateManager.getInstance().getParameters();
		if (parameters!=null)
		{
			for (Map.Entry<String, String> parameter : parameters.entrySet())
			{
				context.put(parameter.getKey(), parameter.getValue());
			}
		}
	}

	private Map<String, String> getSelectionData(DataContext dataContext, VirtualFile directory)
	{
		Map<String, String> selectionData=new HashMap<String, String>();
		selectionData.put("directory", VfsUtil.virtualToIoFile(directory).getAbsolutePath());
		Module module=(Module)dataContext.getData(DataConstants.MODULE);
		if (module!=null)
		{
			selectionData.put("module", module.getName());
			VirtualFile dir=module.getModuleFile().getParent();
			selectionData.put("moduleDirectory", VfsUtil.virtualToIoFile(dir).getAbsolutePath());
		}
		Project project=(Project)dataContext.getData(DataConstants.PROJECT);
		if (project!=null)
		{
			selectionData.put("project", project.getName());
			VirtualFile dir=project.getProjectFile().getParent();
			selectionData.put("projectDirectory", VfsUtil.virtualToIoFile(dir).getAbsolutePath());
		}
		PsiElement psiElement=(PsiElement)dataContext.getData(DataConstants.PSI_ELEMENT);
		PsiPackage psiPackage=getPsiPackage(psiElement);
		if (psiPackage!=null) selectionData.put("package", psiPackage.getQualifiedName());
		PsiDirectory psiRootDirectory=getPsiRootDirectory(psiElement);
		if (psiRootDirectory!=null)
		{
			File rootDirectory=VfsUtil.virtualToIoFile(psiRootDirectory.getVirtualFile());
			selectionData.put("rootDirectory", rootDirectory.getAbsolutePath());
			String relativePath=VfsUtil.getRelativePath(directory, psiRootDirectory.getVirtualFile(), File.separatorChar);
			selectionData.put("relativeSourcePath", relativePath);
		}
		if (psiElement!=null)
		{
			PsiFile psiFile=psiElement.getContainingFile();
			if (psiFile!=null) selectionData.put("file", psiFile.getName());
		}
		return selectionData;
	}

	private void refreshFile(final Project project, final TemplateFile templateFile, final File file)
	{
		ApplicationManager.getApplication().runWriteAction(new Runnable()
		{
			public void run()
			{
				VirtualFile virtualFile=LocalFileSystem.getInstance().refreshAndFindFileByIoFile(file);
				if (virtualFile!=null)
				{
					virtualFile.refresh(true, false);
					if (templateFile.isOpenEditor())
						new OpenFileDescriptor(project, virtualFile).navigate(true);
				}
			}
		});
	}

	private static void handleThrowable(Exception exception, DataContext dataContext)
	{
		exception.printStackTrace();
		JOptionPane.showMessageDialog((Component)dataContext.getData(DataConstants.CONTEXT_COMPONENT),
									  exception.getClass().getSimpleName()+": "+exception.getMessage(),
									  "Unhandled Exception", JOptionPane.ERROR_MESSAGE);
	}

	private static String evaluate(VelocityEngine velocity, VelocityContext context, String template) throws VelocityException, IOException
	{
		StringWriter writer=new StringWriter();
		velocity.evaluate(context, writer, GenerateAction.class.getName(), template);
		return writer.toString();
	}
}
