// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.awt.GridBagConstraints;
import static java.awt.GridBagConstraints.*;
import java.awt.Insets;
import java.util.*;
import javax.swing.*;
import javax.swing.text.JTextComponent;

import org.apache.commons.beanutils.BasicDynaClass;
import org.apache.commons.beanutils.DynaClass;
import org.apache.commons.beanutils.DynaProperty;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.DefaultActionGroup;

import com.kiwisoft.generate.model.*;
import com.kiwisoft.utils.Disposable;
import com.kiwisoft.utils.ListenerSupport;
import com.kiwisoft.utils.ObservableDynaBean;
import com.kiwisoft.utils.gui.lookup.LookupField;
import com.kiwisoft.utils.gui.lookup.TextLookup;

/**
 * A panel for entering values for a collection of primitive elements. 
 *
 * @author Stefan Stiller
 */
class PrimitiveListPanel extends TitledParameterPanel
{
	private ParameterBean bean;
	private Map<ListAdapter, Field> components=new HashMap<ListAdapter, Field>();
	private int row;
	private DynaClass adapterClass;
	private List<ListAdapter> adapters=new ArrayList<ListAdapter>();

	public PrimitiveListPanel(ParameterBean bean, UserParameter parameter)
	{
		super(parameter);
		this.bean=bean;
		adapterClass=new BasicDynaClass("ListAdapter", ListAdapter.class,
										new DynaProperty[]{
											new DynaProperty(ListAdapter.VALUE, parameter.getType().getTypeClass())
										});
		List<Object> list=getList();
		for (int i=0; i<list.size(); i++)
		{
			try
			{
				ListAdapter adapter=new ListAdapter(adapterClass, list.get(i), this);
				adapters.add(adapter);
				addComponent(adapter);
			}
			catch (Exception e)
			{
				throw new RuntimeException(e);
			}
		}
	}

	private List<Object> getList()
	{
		return (List<Object>)this.bean.get(getParameter().getName());
	}

	protected boolean canAddElement()
	{
		return true;
	}

	protected void addElement()
	{
		List<?> list=(List<?>)bean.get(getParameter().getName());
		try
		{
			list.add(null);
			ListAdapter adapter=new ListAdapter(adapterClass, null, this);
			adapters.add(adapter);
			addComponent(adapter);
			getElementsPanel().updateUI();
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
	}

	private void addComponent(ListAdapter adapter)
	{
		Field field=addSimpleParameter(getParameter(), adapter);
		components.put(adapter, field);
	}

	private Field addSimpleParameter(UserParameter parameter, ListAdapter adapter)
	{
		int index=adapter.getIndex();
		JLabel label=new JLabel("#"+(index+1)+":");
		JPanel panel=getElementsPanel();
		panel.add(label, new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		JComponent component=null;
		ListenerSupport listenerSupport=new ListenerSupport();
		Type type=parameter.getType();
		if (type instanceof StringType)
		{
			Set values=parameter.getValues();
			if (values!=null && !values.isEmpty())
			{
				//noinspection unchecked
				component=new LookupField(new TextLookup(values, parameter.isRestricted()));
				panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
				listenerSupport.bind((LookupField)component, adapter, ListAdapter.VALUE);
			}
			else
			{
				component=new JTextField(30);
				panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
				listenerSupport.bind((JTextComponent)component, adapter, ListAdapter.VALUE);
			}
		}
		else if (type instanceof BooleanType)
		{
			component=new JCheckBox();
			panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
			listenerSupport.bind((JCheckBox)component, adapter, ListAdapter.VALUE);
		}
		else if (type instanceof IntegerType || type instanceof DecimalType)
		{
			component=new JFormattedTextField();
			((JFormattedTextField)component).setColumns(8);
			panel.add(component, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 0, 5, 5), 0, 0));
			listenerSupport.bindNumber((JFormattedTextField)component, adapter, ListAdapter.VALUE);
		}
		if (component!=null)
		{
			component.setToolTipText(parameter.getDescription());
			listenerSupport.installFocusListener(component, new ComponentAligner(panel, component));
		}

		DefaultActionGroup actionGroup=new DefaultActionGroup("KiwiCreate.ParameterDialog", false);
		actionGroup.add(new RemoveElementAction(adapter));
		ActionManager actionManager=ActionManager.getInstance();
		JComponent toolbar=actionManager.createActionToolbar("KiwiCreate.ParameterDialog", actionGroup, false).getComponent();
		toolbar.setOpaque(false);
		panel.add(toolbar, new GridBagConstraints(2, row, 1, 1, 0.0, 0.0, EAST, NONE, new Insets(0, 0, 0, 0), 0, 0));

		row++;
		return new Field(label, component, toolbar, listenerSupport);
	}

	protected boolean canRemoveElement(Object value)
	{
		if (value instanceof ListAdapter) return ((ListAdapter)value).canRemove();
		return false;
	}

	protected void removeElement(Object value)
	{
		ListAdapter adapter=(ListAdapter)value;
		Field field=components.remove(adapter);
		JPanel panel=getElementsPanel();
		if (field!=null)
		{
			field.dispose();
			adapters.remove(adapter);
			List<Object> list=getList();
			list.clear();
			for (int i=0; i<adapters.size(); i++)
			{
				ListAdapter listAdapter=adapters.get(i);
				list.add(listAdapter.get(ListAdapter.VALUE));
				components.get(listAdapter).label.setText("#"+(i+1)+":");
			}
			panel.remove(field.label);
			panel.remove(field.component);
			panel.remove(field.toolbar);
			panel.updateUI();
		}
	}

	public void dispose()
	{
		for (Field field : components.values()) field.dispose();
		super.dispose();
	}

	private static class Field implements Disposable
	{
		private JLabel label;
		private JComponent component;
		private JComponent toolbar;
		private ListenerSupport listenerSupport;

		public Field(JLabel label, JComponent component, JComponent toolbar, ListenerSupport listenerSupport)
		{
			this.label=label;
			this.component=component;
			this.toolbar=toolbar;
			this.listenerSupport=listenerSupport;
		}

		public void dispose()
		{
			listenerSupport.dispose();
		}
	}

	public static class ListAdapter extends ObservableDynaBean
	{
		private static final String VALUE="value";
		private PrimitiveListPanel panel;

		public ListAdapter(DynaClass dynaClass)
		{
			super(dynaClass);
			throw new UnsupportedOperationException();
		}

		public ListAdapter(DynaClass dynaClass, Object value, PrimitiveListPanel panel)
		{
			super(dynaClass);
			this.panel=panel;
			super.set(VALUE, value);
		}

		public void set(String name, Object object)
		{
			if (VALUE.equals(name))
			{
				List<Object> list=panel.getList();
				int index=getIndex();
				if (index>=0 && list!=null && index<list.size()) list.set(index, object);
			}
			super.set(name, object);
		}

		public int getIndex()
		{
			return panel.adapters.indexOf(this);
		}

		public boolean canRemove()
		{
			List<Object> list=panel.getList();
			return getIndex()<list.size();
		}
	}
}
