// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.util.*;
import java.util.HashMap;
import java.io.IOException;

import com.kiwisoft.utils.xml.XMLContext;
import com.kiwisoft.utils.xml.XMLObject;
import com.kiwisoft.utils.xml.XMLWriter;

/**
 * A template containing of a parameter model and a list of template files.
 *
 * @author Stefan Stiller
 */
public class Template extends UserType
{
	private List<UserType> types=new ArrayList<UserType>();
	private List<TemplateFile> files=new ArrayList<TemplateFile>();
	private boolean requiresSourceFolder;

	public Template()
	{
	}

	public void addType(UserType type)
	{
		types.add(type);
		fireElementAdded("types", type);
	}

	public void removeType(UserType type)
	{
		types.remove(type);
		fireElementRemoved("types", type);
	}

	public List<UserType> getTypes()
	{
		return Collections.unmodifiableList(types);
	}

	public void addFile(TemplateFile file)
	{
		files.add(file);
		fireElementAdded("files", file);
	}

	public void removeFile(TemplateFile file)
	{
		files.remove(file);
		fireElementRemoved("files", file);
	}

	public List<TemplateFile> getFiles()
	{
		return Collections.unmodifiableList(files);
	}

	public boolean isRequiresSourceFolder()
	{
		return requiresSourceFolder;
	}

	public void setRequiresSourceFolder(boolean requiresSourceFolder)
	{
		boolean oldValue=this.requiresSourceFolder;
		this.requiresSourceFolder=requiresSourceFolder;
		firePropertyChange("requiresSourceFolder", oldValue, requiresSourceFolder);
	}

	// XMLObject interface methods

	public Template(XMLContext context, String aName)
	{
		super(context, aName);
	}

	public void addXMLElement(XMLContext context, XMLObject element)
	{
		if (element instanceof UserType) types.add((UserType)element);
		else if (element instanceof TemplateFile) files.add((TemplateFile)element);
		else super.addXMLElement(context, element);
	}

	public void writeXML(XMLWriter xmlWriter) throws IOException
	{
		xmlWriter.startElement("template");
		xmlWriter.setAttribute("name", getName());
		xmlWriter.setAttribute("requiresSourceFolder", requiresSourceFolder);
		for (UserType group : types) group.writeXML(xmlWriter);
		for (UserParameter parameter : getParameters()) parameter.writeXML(xmlWriter);
		for (TemplateFile file : files) file.writeXML(xmlWriter);
		xmlWriter.closeElement("template");
	}

	public Template copy()
	{
		Template template=new Template();
		template.setRequiresSourceFolder(isRequiresSourceFolder());
		// We need to copy the user types first because we need to reference
		// the new types in the new parameters.
		Map<UserType, UserType> typeMap=new HashMap<UserType, UserType>();
		for (UserType type : getTypes())
		{
			UserType newType=new UserType();
			template.addType(newType);
			typeMap.put(type, newType);
		}
		for (Map.Entry<UserType, UserType> entry : typeMap.entrySet())
		{
			entry.getValue().copyFrom(entry.getKey(), typeMap);
		}
		template.copyFrom(this, typeMap);
		for (TemplateFile file : getFiles()) template.addFile(file.copy());
		return template;
	}

	public void validate()
	{
		super.validate();
		Set<String> names=new HashSet<String>();
		for (UserType type : types)
		{
			type.validate();
			if (names.contains(type.getName()))
				throw new RuntimeException("Duplicate user type '"+type.getName()+"'.");
			names.add(type.getName());
		}
		names.clear();
		for (TemplateFile file : files)
		{
			file.validate();
			if (names.contains(file.getName()))
				throw new RuntimeException("Duplicate template file '"+file.getName()+"'.");
			names.add(file.getName());
		}
	}
}
