// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;
import java.util.Map;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import com.intellij.openapi.application.PathManager;

import com.kiwisoft.utils.xml.XMLHandler;
import com.kiwisoft.utils.xml.XMLWriter;
import com.kiwisoft.utils.IOUtils;

/**
 * Manager for all templates. 
 *
 * @author Stefan Stiller
 */
public class TemplateManager
{
	private static TemplateManager instance;

	public static TemplateManager getInstance()
	{
		if (instance==null)
		{
			instance=new TemplateManager();
			instance.load();
		}
		return instance;
	}

	private Map<String, String> parameters;
	private Set<Template> templates=new HashSet<Template>();

	private TemplateManager()
	{
	}

	private void load()
	{
		File folder=getTemplatePath();
		if (!folder.exists())
		{
			folder.mkdirs();
			try
			{
				IOUtils.copyStream(TemplateManager.class.getResourceAsStream("Bean.zip"),
													  new FileOutputStream(new File(folder, "Bean.zip")));
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}
		if (folder.exists())
		{
			File[] files=folder.listFiles();
			for (File file : files) load(file);
		}
	}

	private void load(File file)
	{
		try
		{
			ZipFile zipFile=new ZipFile(file);
			ZipEntry templateEntry=zipFile.getEntry("template.xml");

			XMLHandler xmlHandler=new XMLHandler();
			xmlHandler.setTrimText(false);
			xmlHandler.addTagMapping("template", Template.class);
			xmlHandler.addTagMapping("parameter", UserParameter.class);
			xmlHandler.addTagMapping("group", UserType.class);
			xmlHandler.addTagMapping("userType", UserType.class);
			xmlHandler.addIdAttribute("group", "name", "types");
			xmlHandler.addIdAttribute("userType", "name", "types");
			xmlHandler.addRefAttribute("parameter", "type", "types");
			for (Type type : Types.getTypes())
			{
				xmlHandler.addStaticIdObject("types", type.getName(), type);
			}
			xmlHandler.addTagMapping("file", TemplateFile.class);

			Template template=(Template)xmlHandler.loadStream(zipFile.getInputStream(templateEntry));
			for (TemplateFile templateFile : template.getFiles())
			{
				ZipEntry fileEntry=zipFile.getEntry(templateFile.getName());
				InputStream inputStream=zipFile.getInputStream(fileEntry);
				StringBuilder content=new StringBuilder();
				byte[] buffer=new byte[1024];
				int length;
				while ((length=inputStream.read(buffer))!=-1) content.append(new String(buffer, 0, length));
				inputStream.close();
				templateFile.setContent(content.toString());
			}
			templates.add(template);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void save()
	{
		File folder=getTemplatePath();
		if (!folder.exists()) folder.mkdirs();
		for (Template template : templates)
		{
			save(getTemplateFile(template), template);
		}
	}

	private File getTemplateFile(Template template)
	{
		return new File(getTemplatePath(), template.getName()+".zip");
	}

	private static File getTemplatePath()
	{
		return new File(PathManager.getConfigPath()+File.separator+"kiwisoft"+File.separator+"templates");
	}

	private void save(File file, Template template)
	{
		try
		{
			ZipOutputStream zipOutputStream=new ZipOutputStream(new FileOutputStream(file));

			zipOutputStream.putNextEntry(new ZipEntry("template.xml"));
			XMLWriter xmlWriter=new XMLWriter(zipOutputStream, null);
			xmlWriter.start();
			template.writeXML(xmlWriter);
			xmlWriter.flush();
			zipOutputStream.closeEntry();

			for (TemplateFile templateFile : template.getFiles())
			{
				zipOutputStream.putNextEntry(new ZipEntry(templateFile.getName()));
				String content=templateFile.getContent();
				if (content!=null) zipOutputStream.write(content.getBytes());
				zipOutputStream.closeEntry();
			}
			zipOutputStream.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	public void addTemplate(Template template)
	{
		templates.add(template);
	}

	public void removeTemplate(Template template)
	{
		templates.remove(template);
		File templateFile=getTemplateFile(template);
		if (templateFile.exists()) templateFile.delete();
	}

	public Set<Template> getTemplates()
	{
		return Collections.unmodifiableSet(templates);
	}

	public void removeAllTemplates()
	{
		for (Template template : new HashSet<Template>(templates))
		{
			removeTemplate(template);
		}
	}

	public void setParameters(Map<String, String> parameters)
	{
		this.parameters=parameters;
	}

	public Map<String, String> getParameters()
	{
		return parameters;
	}
}
