// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.io.IOException;

import org.apache.commons.beanutils.DynaBean;
import org.apache.commons.beanutils.DynaClass;

import com.kiwisoft.utils.*;
import com.kiwisoft.utils.xml.XMLAdapter;
import com.kiwisoft.utils.xml.XMLContext;
import com.kiwisoft.utils.xml.XMLObject;
import com.kiwisoft.utils.xml.XMLWriter;

/**
 * A user defined property type. A type consists of a list of properties. 
 *
 * @author Stefan Stiller
 */
public class UserType extends XMLAdapter implements Type, ObservableBean
{
	private String name;
	private String displayName;
	private String description;
	private List<UserParameter> parameters=new ArrayList<UserParameter>();

	private PropertyChangeSupport changeSupport=new PropertyChangeSupport(this);
	private CollectionChangeSupport collectionSupport=new CollectionChangeSupport(this);

	public UserType()
	{
	}

	public String getName()
	{
		return name;
	}

	public String getDisplayableName()
	{
		if (!StringUtils.isEmpty(displayName)) return displayName;
		return name;
	}

	public String getDisplayName()
	{
		return displayName;
	}

	public void setDisplayName(String displayName)
	{
		String oldDisplayName=this.displayName;
		this.displayName=displayName;
		firePropertyChange("displayName", oldDisplayName, displayName);
	}

	public Class getTypeClass()
	{
		return DynaBean.class;
	}

	public Object parse(String name)
	{
		return null;
	}

	public String format(Object value)
	{
		return null;
	}

	public void setName(String name)
	{
		String oldName=this.name;
		this.name=name;
		firePropertyChange("name", oldName, name);
	}

	public String getDescription()
	{
		return description;
	}

	public void setDescription(String description)
	{
		String oldDescription=this.description;
		this.description=description;
		firePropertyChange("description", oldDescription, description);
	}

	public void addParameter(UserParameter parameter)
	{
		parameters.add(parameter);
		fireElementAdded("parameters", parameter);
	}

	public void removeParameter(UserParameter parameter)
	{
		parameters.remove(parameter);
		fireElementRemoved("parameters", parameter);
	}

	public List<UserParameter> getParameters()
	{
		return Collections.unmodifiableList(parameters);
	}

	public void moveUp(UserParameter parameter)
	{
		int index=parameters.indexOf(parameter);
		if (index>0)
		{
			parameters.remove(parameter);
			parameters.add(index-1, parameter);
		}
	}

	public void moveDown(UserParameter parameter)
	{
		int index=parameters.indexOf(parameter);
		if (index>=0 && index<parameters.size()-1)
		{
			parameters.remove(parameter);
			parameters.add(index+1, parameter);
		}
	}

	// PropertyChangeSupport delegates

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(propertyName, listener);
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(propertyName, listener);
	}

	public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
	{
		changeSupport.firePropertyChange(propertyName, oldValue, newValue);
	}

	// CollectionChangeSupport delegates

	public void addListener(CollectionChangeListener listener)
	{
		collectionSupport.addListener(listener);
	}

	public void removeListener(CollectionChangeListener listener)
	{
		collectionSupport.removeListener(listener);
	}

	public void fireElementAdded(String propertyName, Object newElement)
	{
		collectionSupport.fireElementAdded(propertyName, newElement);
	}

	public void fireElementRemoved(String propertyName, Object oldElement)
	{
		collectionSupport.fireElementRemoved(propertyName, oldElement);
	}

	// XMLObject interface methods

	public UserType(XMLContext context, String name)
	{
		super(context, name);
	}

	public void setXMLAttribute(XMLContext context, String name, String value)
	{
		if ("name".equalsIgnoreCase(name)) this.name=value;
		else if ("displayName".equalsIgnoreCase(name)) this.displayName=value;
		else if ("description".equalsIgnoreCase(name)) this.description=value;
	}

	public void addXMLElement(XMLContext context, XMLObject element)
	{
		if (element instanceof UserParameter) parameters.add((UserParameter)element);
	}

	public DynaClass createDynaClass()
	{
		return new ParameterClass(this);
	}

	public void writeXML(XMLWriter xmlWriter) throws IOException
	{
		xmlWriter.startElement("userType");
		xmlWriter.setAttribute("name", name);
		xmlWriter.setAttribute("displayName", displayName);
		for (UserParameter parameter : getParameters()) parameter.writeXML(xmlWriter);
		xmlWriter.closeElement("userType");
	}

	protected void copyFrom(UserType userType, Map<UserType, UserType> typeMap)
	{
		setName(userType.getName());
		setDisplayName(userType.getDisplayName());
		setDescription(userType.getDescription());
		for (UserParameter parameter : userType.getParameters()) addParameter(parameter.copy(typeMap));
	}

	public void validate()
	{
		if (StringUtils.isEmpty(getName()))
		{
			if (this instanceof Template) throw new RuntimeException("Missing template name.");
			else throw new RuntimeException("Missing type name.");
		}
		if (!(this instanceof Template) && parameters.isEmpty()) throw new RuntimeException("Type doesn't contain any parameters.");
		for (UserParameter parameter : parameters) parameter.validate();
	}

	public boolean isValid()
	{
		return !StringUtils.isEmpty(getName());
	}
}
