// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.awt.Component;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.util.Observable;
import java.util.Observer;
import java.util.Stack;
import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.JTextComponent;

import com.kiwisoft.utils.gui.adapters.*;
import com.kiwisoft.utils.gui.adapters.ComponentAdapter;
import com.kiwisoft.utils.gui.lookup.LookupField;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:50:51 $
 */
public class ListenerSupport implements Disposable
{
	private Stack<Disposable> disposables=new Stack<Disposable>();

	public ListenerSupport()
	{
	}

	public void dispose()
	{
		while (!disposables.isEmpty())
		{
			Disposable disposable=disposables.pop();
			disposable.dispose();
		}
	}

	public void dispose(Disposable disposable)
	{
		if (disposable!=null)
		{
			disposable.dispose();
			disposables.remove(disposable);
		}
	}

	public void addDisposable(Disposable disposable)
	{
		disposables.push(disposable);
	}

	public void installObserver(final Observable observable, final Observer observer)
	{
		observable.addObserver(observer);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				observable.deleteObserver(observer);
			}
		});
	}

	public void installActionListener(final JComboBox comboBox, final ActionListener actionListener)
	{
		comboBox.addActionListener(actionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				comboBox.removeActionListener(actionListener);
			}
		});
	}

	public void installActionListener(final AbstractButton button, final ActionListener actionListener)
	{
		button.addActionListener(actionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				button.removeActionListener(actionListener);
			}
		});
	}

	public void installChangeListener(final JTabbedPane tabbedPane, final ChangeListener changeListener)
	{
		tabbedPane.addChangeListener(changeListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				tabbedPane.removeChangeListener(changeListener);
			}
		});
	}

	public void installMouseListener(final Component component, final MouseListener mouseListener)
	{
		component.addMouseListener(mouseListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeMouseListener(mouseListener);
			}
		});
	}

	public void installKeyListener(final Component component, final KeyListener keyListener)
	{
		component.addKeyListener(keyListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeKeyListener(keyListener);
			}
		});
	}

	public void installDocumentListener(final JTextComponent component, final DocumentListener listener)
	{
		component.getDocument().addDocumentListener(listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.getDocument().removeDocumentListener(listener);
			}
		});
	}

	public void installColumnSelectionListener(final ListSelectionModel selectionModel, final ListSelectionListener selectionListener)
	{
		selectionModel.addListSelectionListener(selectionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				selectionModel.removeListSelectionListener(selectionListener);
			}
		});
	}

	public void installSelectionListener(final JTable table, final ListSelectionListener selectionListener)
	{
		table.getSelectionModel().addListSelectionListener(selectionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				table.getSelectionModel().removeListSelectionListener(selectionListener);
			}
		});
	}

	public void installSelectionListener(final JTree tree, final TreeSelectionListener selectionListener)
	{
		tree.addTreeSelectionListener(selectionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				tree.removeTreeSelectionListener(selectionListener);
			}
		});
	}

	public void installItemListener(final AbstractButton button, final ItemListener itemListener)
	{
		button.addItemListener(itemListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				button.removeItemListener(itemListener);
			}
		});
	}

	public void installPropertyChangeListener(final JComponent component, final PropertyChangeListener listener)
	{
		component.addPropertyChangeListener(listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removePropertyChangeListener(listener);
			}
		});
	}

	public void installPropertyChangeListener(final JComponent component, final String property, final PropertyChangeListener listener)
	{
		component.addPropertyChangeListener(property, listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removePropertyChangeListener(property, listener);
			}
		});
	}

	public void installPropertyChangeListener(final ObservableBean bean, final String property, final PropertyChangeListener listener)
	{
		bean.addPropertyChangeListener(property, listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				bean.removePropertyChangeListener(property, listener);
			}
		});
	}

	public void installComponentEnabler(final JToggleButton button, final Object[] components)
	{
		enableComponents(components, button.isSelected());
		installItemListener(button, new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				boolean selected=e.getStateChange()==ItemEvent.SELECTED;
				enableComponents(components, selected);
			}
		});
	}

	private void enableComponents(Object[] components, boolean enable)
	{
		for (int i=0; i<components.length; i++)
		{
			if (components[i] instanceof Component)
			{
				((Component)components[i]).setEnabled(enable);
			}
			else if (components[i] instanceof Action)
			{
				((Action)components[i]).setEnabled(enable);
			}
		}
	}

	public ComponentAdapter bind(JTextComponent component, ObservableBean object, String property)
	{
		return bind(new TextComponentAdapter(component), object, property);
	}

	public ComponentAdapter bind(LookupField lookupField, ObservableBean object, String property)
	{
		return bind(new LookupAdapter(lookupField), object, property);
	}

	public ComponentAdapter bind(JCheckBox component, ObservableBean object, String property)
	{
		return bind(new CheckBoxAdapter(component), object, property);
	}

	public ComponentAdapter bindNumber(JFormattedTextField component, ObservableBean bean, String property)
	{
		return bind(new NumberAdapter(component), bean, property);
	}

	private ComponentAdapter bind(final ComponentAdapter componentAdapter, ObservableBean object, String property)
	{
		try
		{
			componentAdapter.bind(object, property);
			disposables.push(new Disposable()
			{
				public void dispose()
				{
					componentAdapter.unbind();
				}
			});
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);
		}
		return componentAdapter;
	}

	public void installFocusListener(final JComponent component, final FocusListener listener)
	{
		component.addFocusListener(listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeFocusListener(listener);
			}
		});
	}
}
