// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.adapters;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.lang.reflect.InvocationTargetException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import javax.swing.JFormattedTextField;
import javax.swing.SwingConstants;
import javax.swing.text.NumberFormatter;
import javax.swing.text.DefaultFormatterFactory;

import org.apache.commons.beanutils.PropertyUtils;

import com.kiwisoft.utils.ObservableBean;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.StringUtils;

/**
 * An adapter binding a text field to a number property. 
 *
 * @author Stefan Stiller
 */
public class NumberAdapter implements PropertyChangeListener, ComponentAdapter
{
	private JFormattedTextField component;
	private ObservableBean object;
	private String property;

	public NumberAdapter(JFormattedTextField component)
	{
		this.component=component;
		component.setHorizontalAlignment(SwingConstants.TRAILING);
	}

	public void bind(ObservableBean object, String property) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException
	{
		this.object=object;
		this.property=property;
		boolean readable=false;
		boolean writeable=false;
		component.setText("");
		if (object!=null)
		{
			readable=PropertyUtils.isReadable(object, property);
			Class type=PropertyUtils.getPropertyType(object, property);
			if (!Number.class.isAssignableFrom(type))
				throw new RuntimeException("The property '"+property+"' is no number property.");
			NumberFormatter formatter=new NumberFormatter(getJavaNumberFormat())
			{
				public Object stringToValue(String text) throws ParseException
				{
					if (StringUtils.isEmpty(text)) return null;
					return super.stringToValue(text);
				}
			};
			formatter.setValueClass(type);
			component.setFormatterFactory(new DefaultFormatterFactory(formatter));
			try
			{
				writeable=readable && PropertyUtils.isWriteable(object, property);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			if (readable)
			{
				updateComponent();
				object.addPropertyChangeListener(property, this);
			}
			if (writeable)
			{
				component.addPropertyChangeListener("value", this);
			}
		}
		component.setEnabled(readable);
		component.setEditable(writeable);
	}

	private void updateComponent()
	{
		try
		{
			Object value=PropertyUtils.getProperty(object, property);
			component.setValue(value);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void unbind()
	{
		if (object!=null) object.removePropertyChangeListener(property, this);
		component.removePropertyChangeListener("value", this);
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		if (evt.getSource()==component)
		{
			if (object!=null && component.isEnabled())
			{
				try
				{
					Object oldValue=PropertyUtils.getProperty(object, property);
					if (!Utils.equals(oldValue, component.getValue()))
						PropertyUtils.setProperty(object, property, component.getValue());
				}
				catch (Exception e1)
				{
					e1.printStackTrace();
				}
			}
		}
		else
		{
			updateComponent();
		}
	}

	public static NumberFormat getJavaNumberFormat()
	{
		NumberFormat numberFormat=NumberFormat.getInstance(Locale.US);
		numberFormat.setMinimumFractionDigits(0);
		numberFormat.setMaximumFractionDigits(8);
		numberFormat.setMinimumIntegerDigits(1);
		numberFormat.setMaximumIntegerDigits(8);
		numberFormat.setGroupingUsed(false);
		numberFormat.setParseIntegerOnly(false);
		return numberFormat;
	}

	public boolean stopEditing()
	{
		try
		{
			component.commitEdit();
		}
		catch (ParseException e)
		{
			return false;
		}
		return true;
	}
}
