// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.awt.Component;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;
import java.sql.Timestamp;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;

import com.kiwisoft.utils.DoubleKeyMap;
import com.kiwisoft.utils.Tristate;
import com.kiwisoft.utils.ClassFeatureManager;
import com.kiwisoft.utils.format.*;
import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.gui.lookup.DialogLookup;
import com.kiwisoft.utils.gui.lookup.Lookup;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.8 $, $Date: 2006/03/24 18:58:21 $
 */
public class TableEditorFactory extends ClassFeatureManager<ObjectEditor>
{
	private static TableEditorFactory instance;

	public synchronized static TableEditorFactory getInstance()
	{
		if (instance==null) instance=new TableEditorFactory();
		return instance;
	}

	private DoubleKeyMap<ObjectEditor, ObjectStyle, StyleEditor> styleEditors=new DoubleKeyMap<ObjectEditor, ObjectStyle, StyleEditor>();

	private TableEditorFactory()
	{
		setFeature(String.class, new TableTextEditor(String.class, new DefaultStringFormat(DEFAULT)));

		DefaultNumberFormat numberFormat=new DefaultNumberFormat(DEFAULT);
		setFeature(Byte.class, new DefaultObjectEditor(Byte.class, numberFormat));
		setFeature(Short.class, new DefaultObjectEditor(Short.class, numberFormat));
		setFeature(Integer.class, new DefaultObjectEditor(Integer.class, numberFormat));
		setFeature(Long.class, new DefaultObjectEditor(Long.class, numberFormat));
		setFeature(BigInteger.class, new DefaultObjectEditor(BigInteger.class, numberFormat));
		setFeature(Float.class, new DefaultObjectEditor(Float.class, numberFormat));
		setFeature(Double.class, new DefaultObjectEditor(Double.class, numberFormat));
		setFeature(BigDecimal.class, new DefaultObjectEditor(BigDecimal.class, numberFormat));

		setFeature(Boolean.class, new BooleanObjectEditor());
		setFeature(Tristate.class, new TristateObjectEditor(false));

		setFeature(Date.class, new TableDateEditor(Date.class, new DefaultDateFormat(DEFAULT)));
		setFeature(Timestamp.class, new TableDateEditor(Timestamp.class, new DefaultDateFormat(DEFAULT)));
		setFeature(java.sql.Date.class, new TableDateEditor(java.sql.Date.class, new DefaultDateFormat(DEFAULT)));

		setFeature(Boolean.class, "Tristate", new TristateObjectEditor(true));
	}

	public ObjectEditor getEditor(Class aClass, String variant)
	{
		ObjectEditor editor=getFeature(aClass, variant);
		if (editor!=null) return editor.cloneEditor();
		return null;
	}

	protected ObjectEditor getInheritedDefaultFeature(Class aClass)
	{
		ObjectFormat format=FormatManager.getInstance().getRegisteredFormat(aClass);
		if (format!=null && format.canParse(aClass)) return createFormatEditor(format, aClass);
		return super.getInheritedDefaultFeature(aClass);
	}

	private ObjectEditor createFormatEditor(ObjectFormat format, Class aClass)
	{
		if (format instanceof BooleanFormat)
			return new BooleanObjectEditor(aClass, (BooleanFormat)format);
		else if (Date.class.isAssignableFrom(aClass))
			return new TableDateEditor(aClass, (TextFormat)format);
		else
			return new DefaultObjectEditor(aClass, (TextFormat)format);
	}

	protected ObjectEditor getInheritedVariantFeature(Class aClass, String variant)
	{
		ObjectFormat format=FormatManager.getInstance().getRegisteredFormat(aClass, variant);
		if (format!=null && format.canParse(aClass)) return createFormatEditor(format, aClass);
		return super.getInheritedVariantFeature(aClass, variant);
	}

	public TableCellEditor getEditor(Class aClass, String variant, ObjectStyle style)
	{
		ObjectEditor editor=getEditor(aClass, variant);
		if (editor==null) return editor;
		return wrapEditor(editor, style, true);
	}

	public TableCellEditor wrapEditor(ObjectEditor editor, ObjectStyle style, boolean cache)
	{
		if (style==null) return editor;
		StyleEditor styleEditor=styleEditors.get(editor, style);
		if (styleEditor==null)
		{
			styleEditor=new StyleEditor(editor, style);
			if (cache) styleEditors.put(editor, style, styleEditor);
		}
		else styleEditor=styleEditor.cloneEditor();
		return styleEditor;
	}

	public void setEditor(Class aClass, ObjectEditor editor)
	{
		setFeature(aClass, editor);
	}

	public void setEditor(Class aClass, Lookup lookup)
	{
		setEditor(aClass, new TableLookupEditor(lookup));
	}

	public void setEditor(Class aClass, String variant, ObjectEditor editor)
	{
		setFeature(aClass, variant, editor);
	}

	public void setEditor(Class aClass, String variant, DialogLookup lookup)
	{
		setFeature(aClass, variant, new TableDialogLookupEditor(lookup));
	}

	static class StyleEditor implements TableCellEditor
	{
		private ObjectEditor editor;
		private ObjectStyle style;

		private StyleEditor(ObjectEditor editor, ObjectStyle style)
		{
			this.editor=editor.cloneEditor();
			this.style=style;
		}

		void setQuickEditable(boolean editable)
		{
			editor.setQuickEditable(editable);
		}

		public void removeCellEditorListener(CellEditorListener l)
		{
			editor.removeCellEditorListener(l);
		}

		public boolean isCellEditable(EventObject anEvent)
		{
			return editor.isCellEditable(anEvent);
		}

		public void addCellEditorListener(CellEditorListener l)
		{
			editor.addCellEditorListener(l);
		}

		public Object getCellEditorValue()
		{
			return editor.getCellEditorValue();
		}

		public void cancelCellEditing()
		{
			editor.cancelCellEditing();
		}

		public boolean shouldSelectCell(EventObject anEvent)
		{
			return editor.shouldSelectCell(anEvent);
		}

		public boolean stopCellEditing()
		{
			return editor.stopCellEditing();
		}

		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
		{
			if (style.getForeground()!=null)
				editor.setForeground(style.getForeground());
			if (style.getBackground()!=null)
				editor.setBackground(style.getBackground());
			return editor.getTableCellEditorComponent(table, value, isSelected, row, column);
		}

		public StyleEditor cloneEditor()
		{
			return new StyleEditor(editor, style);
		}
	}

}
