// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.tree;

import java.awt.Color;
import java.awt.Font;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.gui.ObjectStyle;

/**
 * A TreeNode implementation with support of Java 5 features. 
 *
 * @author Stefan Stiller
 */
public class GenericTreeNode <T> implements TreeNode
{
	public final static ObjectStyle INVALID_STYLE=new ObjectStyle(Color.RED, null);

	protected DefaultTreeModel treeModel;

	/*
	 * this node's parent, or null if this node has no parent
	 */
	private GenericTreeNode parent;

	/**
	 * array of children, may be null if this node has no children
	 */
	private Vector<GenericTreeNode> children;

	/**
	 * optional user object
	 */
	private T userObject;

	/**
	 * Creates a tree node with no parent, no children, but which allows
	 * children, and initializes it with the specified user object.
	 *
	 * @param userObject an Object provided by the user that constitutes
	 *                   the node's data
	 */
	public GenericTreeNode(T userObject)
	{
		this.userObject=userObject;
	}

	/**
	 * Creates a tree node with no parent, and the given children.
	 *
	 * @param userObject an Object provided by the user that constitutes
	 *                   the node's data
	 */
	public GenericTreeNode(T userObject, GenericTreeNode... children)
	{
		this.userObject=userObject;
		this.children=new Vector<GenericTreeNode>();
		if (children!=null) Collections.addAll(this.children, children);
	}

	public void addChild(GenericTreeNode newChild, int childIndex)
	{
		if (children!=null)
		{
			assert newChild!=null;
			GenericTreeNode oldParent=newChild.getParent();
			if (oldParent!=null) oldParent.removeChild(newChild);
			newChild.setParent(this);
			newChild.setTreeModel(treeModel);
			if (children==null) children=new Vector<GenericTreeNode>();
			children.add(childIndex, newChild);
			fireChildAdded(childIndex);
		}
	}

	protected void fireChildAdded(int childIndex)
	{
		treeModel.nodesWereInserted(this, new int[]{childIndex});
	}

	/**
	 * Removes the child at the specified index from this node's children
	 * and sets that node's parent to null. The child node to remove
	 * must be a <code>MutableTreeNode</code>.
	 *
	 * @param	childIndex	the index in this node's child array
	 * of the child to remove
	 * @exception	ArrayIndexOutOfBoundsException	if <code>childIndex</code> is out of bounds
	 */
	public void removeChild(int childIndex)
	{
		if (children!=null)
		{
			GenericTreeNode child=getChildAt(childIndex);
			children.remove(childIndex);
			child.dispose();
			fireChildRemoved(childIndex, child);
		}
	}

	private void fireChildRemoved(int childIndex, GenericTreeNode child)
	{
		treeModel.nodesWereRemoved(this, new int[]{childIndex}, new Object[]{child});
	}

	public void setParent(GenericTreeNode parent)
	{
		this.parent=parent;
	}

	/**
	 * Returns this node's parent or null if this node has no parent.
	 *
	 * @return	this node's parent TreeNode, or null if this node has no parent
	 */
	public GenericTreeNode getParent()
	{
		return parent;
	}

	/**
	 * Returns the child at the specified index in this node's child array.
	 *
	 * @param	index	an index into this node's child array
	 * @exception	ArrayIndexOutOfBoundsException	if <code>index</code>
	 * is out of bounds
	 * @return	the TreeNode in this node's child array at  the specified index
	 */
	public GenericTreeNode getChildAt(int index)
	{
		return getChildren().get(index);
	}

	/**
	 * Returns the number of children of this node.
	 *
	 * @return	an int giving the number of children of this node
	 */
	public int getChildCount()
	{
		return getChildren().size();
	}

	private Vector<GenericTreeNode> getChildren()
	{
		if (children==null)
		{
			children=loadChildren();
			for (GenericTreeNode child : children)
			{
				child.setParent(this);
				child.setTreeModel(treeModel);
			}
		}
		return children;
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		return new Vector<GenericTreeNode>();
	}

	/**
	 * Returns the index of the specified child in this node's child array.
	 * If the specified node is not a child of this node, returns
	 * <code>-1</code>.  This method performs a linear search and is O(n)
	 * where n is the number of children.
	 *
	 * @param	child	the TreeNode to search for among this node's children
	 * @exception	IllegalArgumentException	if <code>aChild</code>
	 * is null
	 * @return	an int giving the index of the node in this node's child
	 * array, or <code>-1</code> if the specified node is a not
	 * a child of this node
	 */
	@SuppressWarnings({"SuspiciousMethodCalls"})
	public int getIndex(TreeNode child)
	{
		assert child!=null;
		return getChildren().indexOf(child);
	}

	/**
	 * Creates and returns a forward-order enumeration of this node's
	 * children.  Modifying this node's child array invalidates any child
	 * enumerations created before the modification.
	 *
	 * @return	an Enumeration of this node's children
	 */
	public Enumeration children()
	{
		return getChildren().elements();
	}

	/**
	 * Returns true if this node is allowed to have children.
	 *
	 * @return	true if this node allows children, else false
	 */
	public boolean getAllowsChildren()
	{
		return true;
	}

	/**
	 * Sets the user object for this node to <code>userObject</code>.
	 *
	 * @param	userObject	the Object that constitutes this node's
	 * user-specified data
	 * @see	#getUserObject
	 * @see	#toString
	 */
	public void setUserObject(T userObject)
	{
		this.userObject=userObject;
	}

	/**
	 * Returns this node's user object.
	 *
	 * @return	the Object stored at this node by the user
	 * @see	#setUserObject
	 * @see	#toString
	 */
	public T getUserObject()
	{
		return userObject;
	}

	/**
	 * Removes <code>child</code> from this node's child array, giving it a
	 * null parent.
	 *
	 * @param	child	a child of this node to remove
	 * @exception	IllegalArgumentException	if <code>child</code>
	 * is null or is not a child of this node
	 */
	public void removeChild(GenericTreeNode child)
	{
		assert child!=null;
		removeChild(getIndex(child));	// linear search
	}

	public void removeObject(Object object)
	{
		if (children!=null)
		{
			List<GenericTreeNode> nodes=new ArrayList<GenericTreeNode>();
			for (int i=0; i<children.size(); i++)
			{
				GenericTreeNode childNode=children.get(i);
				if (object.equals(childNode.getUserObject())) nodes.add(childNode);
			}
			for (GenericTreeNode node : nodes) removeChild(node);
		}
	}

	/**
	 * Removes all of this node's children, setting their parents to null.
	 * If this node has no children, this method does nothing.
	 */
	public void removeAllChildren()
	{
		for (int i=getChildCount()-1; i>= 0; i--)
		{
			removeChild(i);
		}
	}

	/**
	 * Removes <code>newChild</code> from its parent and makes it a child of
	 * this node by adding it to the end of this node's child array.
	 *
	 * @param	newChild	node to add as a child of this node
	 * @exception	IllegalArgumentException if <code>newChild</code>
	 * is null
	 * @exception	IllegalStateException	if this node does not allow
	 * children
	 */
	public void addChild(GenericTreeNode newChild)
	{
		if (children!=null)
		{
			if (newChild!=null && newChild.getParent()==this)
				addChild(newChild, getChildCount()-1);
			else
				addChild(newChild, getChildCount());
		}
	}

	/**
	 * Returns true if this node has no children.  To distinguish between
	 * nodes that have no children and nodes that <i>cannot</i> have
	 * children (e.g. to distinguish files from empty directories), use this
	 * method in conjunction with <code>getAllowsChildren</code>
	 *
	 * @see	#getAllowsChildren
	 * @return	true if this node has no children
	 */
	public boolean isLeaf()
	{
		return children!=null && children.size()==0;
	}

	public String getText()
	{
		return String.valueOf(getUserObject());
	}

	public ImageIcon getIcon()
	{
		return null;
	}

	public ImageIcon getExpandedIcon()
	{
		return getIcon();
	}

	public JComponent[] getPopupMenu()
	{
		return null;
	}

	void setTreeModel(DefaultTreeModel treeModel)
	{
		this.treeModel=treeModel;
		if (children!=null)
		{
			for (GenericTreeNode child : children) child.setTreeModel(treeModel);
		}
	}

	public Action[] getActions(GenericTree genericTree)
	{
		return null;
	}

	protected void fireNodeChanged()
	{
		if (treeModel!=null)
		{
			treeModel.nodeChanged(this);
		}
	}

	public void dispose()
	{
		setParent(null);
		setTreeModel(null);
		if (children!=null)
		{
			for (GenericTreeNode child : children) child.dispose();
		}
	}

	public GenericTreeNode getChildNode(Object object)
	{
		for (GenericTreeNode child : getChildren())
		{
			if (Utils.equals(object, child.getUserObject())) return child;
		}
		return null;
	}

	public ObjectStyle getStyle()
	{
		return null;
	}

	public boolean canMoveUp(GenericTreeNode childNode)
	{
		return getChildren().indexOf(childNode)>0;
	}

	public boolean canMoveDown(GenericTreeNode childNode)
	{
		int index=getChildren().indexOf(childNode);
		return index>=0 && index<getChildCount()-1;
	}

	public boolean moveUp(GenericTreeNode childNode)
	{
		int index=getChildren().indexOf(childNode);
		if (index>0)
		{
			getChildren().remove(childNode);
			getChildren().add(index-1, childNode);
			fireStructureChanged();
			return true;
		}
		return false;
	}

	public boolean moveDown(GenericTreeNode childNode)
	{
		int index=getChildren().indexOf(childNode);
		if (index>=0 && index<getChildCount()-1)
		{
			getChildren().remove(childNode);
			getChildren().add(index+1, childNode);
			fireStructureChanged();
			return true;
		}
		return false;
	}

	protected void fireStructureChanged()
	{
		if (treeModel!=null) treeModel.nodeStructureChanged(this);
	}

	public boolean canMoveUp()
	{
		GenericTreeNode parent=getParent();
		return parent!=null && parent.canMoveUp(this);
	}

	public boolean canMoveDown()
	{
		GenericTreeNode parent=getParent();
		return parent!=null && parent.canMoveDown(this);
	}

	public void moveUp()
	{
		GenericTreeNode parent=getParent();
		if (parent!=null) parent.moveUp(this);
	}

	public void moveDown()
	{
		GenericTreeNode parent=getParent();
		if (parent!=null) parent.moveDown(this);
	}

	public Font getFont(JTree tree)
	{
		return null;
	}
}
